@@ -81,7 +81,7 @@ class OpenADRClient:
         self.client_session = None
         self.report_queue_task = None
 
-        self.received_events = {}               # Holds the events that we received.
+        self.received_events = []               # Holds the events that we received.
         self.responded_events = {}              # Holds the events that we already saw.
 
         self.cert_path = cert
@@ -133,7 +133,7 @@ class OpenADRClient:
         # Set up automatic polling
         if self.poll_frequency > timedelta(hours=24):
             logger.warning("Polling with intervals of more than 24 hours is not supported. "
-                           "Will use 24 hours as the logging interval.")
+                           "Will use 24 hours as the polling interval.")
             self.poll_frequency = timedelta(hours=24)
         cron_config = utils.cron_config(self.poll_frequency, randomize_seconds=self.allow_jitter)
 
@@ -177,7 +177,8 @@ class OpenADRClient:
                    reading_type=enums.READING_TYPE.DIRECT_READ,
                    report_type=enums.REPORT_TYPE.READING, sampling_rate=None, data_source=None,
                    scale="none", unit=None, power_ac=True, power_hertz=50, power_voltage=230,
-                   market_context=None, end_device_asset_mrid=None, report_data_source=None):
+                   market_context=None, end_device_asset_mrid=None, report_data_source=None,
+                   on_change=False):
         """
         Add a new reporting capability to the client.
 
@@ -215,6 +216,10 @@ class OpenADRClient:
         :param str market_context: The Market Context that this report belongs to.
         :param str end_device_asset_mrid: the Meter ID for the end device that is measured by this report.
         :param report_data_source: A (list of) target(s) that this report is related to.
+        :param boolean on_change: Whether this measurement supports reporting when the value changes.
+                                  To use this functionality, call the ``client.report_on_change`` method
+                                  with parameters ``report_id, r_id, value``. You get the ``report_id``
+                                  and the ``r_id`` from the return of the ``client.add_report`` call.
         """
 
         # Verify input
@@ -237,11 +242,11 @@ class OpenADRClient:
         if sampling_rate is None:
             sampling_rate = objects.SamplingRate(min_period=timedelta(seconds=10),
                                                  max_period=timedelta(hours=24),
-                                                 on_change=False)
+                                                 on_change=on_change)
         elif isinstance(sampling_rate, timedelta):
             sampling_rate = objects.SamplingRate(min_period=sampling_rate,
                                                  max_period=sampling_rate,
-                                                 on_change=False)
+                                                 on_change=on_change)
 
         if data_collection_mode not in ('incremental', 'full'):
             raise ValueError("The data_collection_mode should be 'incremental' or 'full'.")
@@ -408,7 +413,7 @@ class OpenADRClient:
     #                                                                         #
     ###########################################################################
 
-    async def request_event(self, reply_limit=1):
+    async def request_event(self, reply_limit=None):
         """
         Request the next Event from the VTN, if it has any.
         """
@@ -420,7 +425,7 @@ class OpenADRClient:
         response_type, response_payload = await self._perform_request(service, message)
         return response_type, response_payload
 
-    async def created_event(self, request_id, event_id, opt_type, modification_number=1):
+    async def created_event(self, request_id, event_id, opt_type, modification_number=0):
         """
         Inform the VTN that we created an event.
         """
@@ -550,6 +555,11 @@ class OpenADRClient:
         Create a single report in response to a request from the VTN.
         """
 
+    def update_report_on_change(self, report_id, r_id, value):
+        """
+        Creates a new report after a value changed.
+        """
+
     async def update_report(self, report_request_id):
         """
         Call the previously registered report callback and send the result as a message to the VTN.
@@ -682,7 +692,8 @@ class OpenADRClient:
         """
         Placeholder for the on_update_event handler.
         """
-        logger.warning("You should implement your own on_update_event handler. This handler receives "
+        logger.warning("An Event was updated, but you don't have an on_updated_event handler configured. "
+                       "You should implement your own on_update_event handler. This handler receives "
                        "an Event dict and should return either 'optIn' or 'optOut' based on your "
                        "choice. Will re-use the previous opt status for this event_id for now")
         if event['event_descriptor']['event_id'] in self.events:
@@ -752,21 +763,25 @@ class OpenADRClient:
                 event_id = event['event_descriptor']['event_id']
                 event_status = event['event_descriptor']['event_status']
                 modification_number = event['event_descriptor']['modification_number']
-                if event_id in self.received_events:
-                    if self.received_events[event_id]['event_descriptor']['modification_number'] == modification_number:
+                received_event = utils.find_by(self.received_events, 'event_descriptor.event_id', event_id)
+                if received_event:
+                    if received_event['event_descriptor']['modification_number'] == modification_number:
                         # Re-submit the same opt type as we already had previously
                         result = self.responded_events[event_id]
                     else:
+                        # Replace the event with the fresh copy
+                        utils.pop_by(self.received_events, 'event_descriptor.event_id', event_id)
+                        self.received_events.append(event)
                         # Wait for the result of the on_update_event handler
-                        result = self.on_update_event(event)
+                        result = await utils.await_if_required(self.on_update_event(event))
                 else:
                     # Wait for the result of the on_event
-                    self.received_events[event_id] = event
+                    self.received_events.append(event)
                     result = self.on_event(event)
                 if asyncio.iscoroutine(result):
                     result = await result
                 results.append(result)
-                if event_status == 'completed':
+                if event_status in (enums.EVENT_STATUS.COMPLETED, enums.EVENT_STATUS.CANCELLED):
                     self.responded_events.pop(event_id)
                 else:
                     self.responded_events[event_id] = result
@@ -784,7 +799,7 @@ class OpenADRClient:
                             'response_description': 'OK',
                             'opt_type': results[i],
                             'request_id': message['request_id'],
-                            'modification_number': 1,
+                            'modification_number': modification_number,
                             'event_id': events[i]['event_descriptor']['event_id']}
                            for i, event in enumerate(events)
                            if event['response_required'] == 'always'
@@ -806,13 +821,13 @@ class OpenADRClient:
 
     async def _event_cleanup(self):
         """
-        Periodic task that will clean up completed events in our memory.
+        Periodic task that will clean up completed and cancelled events in our memory.
         """
-        print("Checking for stale events")
-        for event in list(self.received_events):
-            if utils.determine_event_status(self.received_events[event]['active_period']) == 'completed':
-                logger.debug(f"Removing event {event} because it is completed.")
-                self.received_events.pop(event)
+        for event in self.received_events:
+            if event['event_descriptor']['event_status'] == 'cancelled' or \
+                    utils.determine_event_status(event['active_period']) == 'completed':
+                logger.info(f"Removing event {event} because it is no longer relevant.")
+                self.received_events.pop(self.received_events.index(event))
 
     async def _poll(self):
         logger.debug("Now polling for new messages")
@@ -829,7 +844,7 @@ class OpenADRClient:
             await self.create_party_registration()
 
         if response_type == 'oadrDistributeEvent':
-            if len(response_payload['events']) > 0:
+            if 'events' in response_payload and len(response_payload['events']) > 0:
                 await self._on_event(response_payload)
 
         elif response_type == 'oadrUpdateReport':
