@@ -75,6 +75,8 @@ class VTNService:
                 # Add the request fingerprint to the message so that the handler can check for it.
                 if request.secure and message_type == 'oadrCreatePartyRegistration':
                     message_payload['fingerprint'] = get_cert_fingerprint_from_request(request)
+                if message_type == 'oadrCreatePartyRegistration':
+                    message_payload['ip_address'] = request.remote
                 response_type, response_payload = await self.handle_message(message_type,
                                                                             message_payload)
             except Exception as err:
