@@ -404,6 +404,7 @@ def extract_pem_cert(tree):
 def find_by(dict_or_list, key, value, *args):
     """
     Find a dict inside a dict or list by key, value properties.
+    You can search for a nesting by separating the levels with a period (.).
     """
     search_params = [(key, value)]
     if args:
@@ -411,16 +412,19 @@ def find_by(dict_or_list, key, value, *args):
     if isinstance(dict_or_list, dict):
         dict_or_list = dict_or_list.values()
     for item in dict_or_list:
-        if not isinstance(item, dict):
-            _item = item.__dict__
-        else:
-            _item = item
         for key, value in search_params:
+            _item = item
+            keys = key.split(".")
+            for key in keys[:-1]:
+                if not hasmember(_item, key):
+                    break
+                _item = getmember(_item, key)
+            key = keys[-1]
             if isinstance(value, tuple):
-                if key not in _item or _item[key] not in value:
+                if not hasmember(_item, key) or getmember(_item, key) not in value:
                     break
             else:
-                if key not in _item or _item[key] != value:
+                if not hasmember(_item, key) or getmember(_item, key) != value:
                     break
         else:
             return item
@@ -444,6 +448,17 @@ def group_by(list_, key, pop_key=False):
     return grouped
 
 
+def pop_by(list_, key, value, *args):
+    """
+    Pop the first item that satisfies the search params from the given list.
+    """
+    item = find_by(list_, key, value, *args)
+    if item:
+        index = list_.index(item)
+        list_.pop(index)
+    return item
+
+
 def cron_config(interval, randomize_seconds=False):
     """
     Returns a dict with cron settings for the given interval
@@ -578,7 +593,7 @@ def determine_event_status(active_period):
         return 'completed'
     if now >= active_period_start:
         return 'active'
-    if getmember(active_period, 'ramp_up_period', None) is not None:
+    if getmember(active_period, 'ramp_up_period', missing=None) is not None:
         ramp_up_start = active_period_start - getmember(active_period, 'ramp_up_period')
         if now >= ramp_up_start:
             return 'near'
@@ -615,44 +630,40 @@ def hasmember(obj, member):
 
 def getmember(obj, member, missing='_RAISE_'):
     """
-    Get a member from a dict or dataclass
+    Get a member from a dict or dataclass. Nesting is possible.
     """
-    if is_dataclass(obj):
-        if not missing == '_RAISE_' and not hasattr(obj, member):
-            return missing
+    def getmember_inner(obj, member, missing='_RAISE_'):
+        if is_dataclass(obj):
+            if not missing == '_RAISE_' and not hasattr(obj, member):
+                return missing
+            else:
+                return getattr(obj, member)
         else:
-            return getattr(obj, member)
-    else:
-        if missing == '_RAISE_':
-            return obj[member]
-        else:
-            return obj.get(member, missing)
+            if missing == '_RAISE_':
+                return obj[member]
+            else:
+                return obj.get(member, missing)
+
+    for m in member.split("."):
+        obj = getmember_inner(obj, m, missing=missing)
+    return obj
 
 
 def setmember(obj, member, value):
     """
     Set a member of a dict of dataclass
     """
+    if '.' in member:
+        members = member.split('.')
+        obj = getmember(obj, ".".join(members[:-1]))
+        member = members[-1]
+
     if is_dataclass(obj):
         setattr(obj, member, value)
     else:
         obj[member] = value
 
 
-def get_next_event_from_deque(deque):
-    unused_elements = []
-    event = None
-    for i in range(len(deque)):
-        msg = deque.popleft()
-        if isinstance(msg, objects.Event) or (isinstance(msg, dict) and 'event_descriptor' in msg):
-            event = msg
-            break
-        else:
-            unused_elements.append(msg)
-    deque.extend(unused_elements)
-    return event
-
-
 def validate_report_request_tuples(list_of_report_requests, mode='full'):
     if len(list_of_report_requests) == 0:
         return
@@ -762,7 +773,7 @@ def order_events(events, limit=None, offset=None):
     """
     def event_priority(event):
         # The default and lowest priority is 0, which we should interpret as a high value.
-        priority = getmember(getmember(event, 'event_descriptor'), 'priority', float('inf'))
+        priority = getmember(event, 'event_descriptor.priority', missing=float('inf'))
         if priority == 0:
             priority = float('inf')
         return priority
@@ -776,27 +787,39 @@ def order_events(events, limit=None, offset=None):
 
     # Update the event statuses
     for event in events:
-        event_status = determine_event_status(getmember(event, 'active_period'))
-        setmember(getmember(event, 'event_descriptor'), 'event_status', event_status)
+        if getmember(event, 'event_descriptor.event_status') != enums.EVENT_STATUS.CANCELLED:
+            event_status = determine_event_status(getmember(event, 'active_period'))
+            setmember(event, 'event_descriptor.event_status', event_status)
 
     # Short circuit if we only have one event:
     if len(events) == 1:
         return events
 
     # Get all the active events first
-    active_events = [event for event in events if getmember(getmember(event, 'event_descriptor'), 'event_status') == 'active']
-    other_events = [event for event in events if getmember(getmember(event, 'event_descriptor'), 'event_status') != 'active']
+    active_events = [event for event in events
+                     if getmember(event, 'event_descriptor.event_status') == 'active']
+    other_events = [event for event in events
+                    if getmember(event, 'event_descriptor.event_status') != 'active']
 
     # Sort the active events by priority
     active_events.sort(key=lambda e: event_priority(e))
 
     # Sort the active events by start date
-    active_events.sort(key=lambda e: getmember(getmember(e, 'active_period'), 'dtstart'))
+    active_events.sort(key=lambda e: getmember(e, 'active_period.dtstart'))
 
     # Sort the non-active events by their start date
-    other_events.sort(key=lambda e: getmember(getmember(e, 'active_period'), 'dtstart'))
+    other_events.sort(key=lambda e: getmember(e, 'active_period.dtstart'))
 
     ordered_events = active_events + other_events
     if limit and offset:
         return ordered_events[offset:offset+limit]
     return ordered_events
+
+
+def increment_event_modification_number(event):
+    """
+    Increments the modification number of the event by 1 and returns the new modification number.
+    """
+    modification_number = getmember(event, 'event_descriptor.modification_number') + 1
+    setmember(event, 'event_descriptor.modification_number', modification_number)
+    return modification_number
